% One step ahead linear predictor for scalar time series
% Uses Durbin-Levinson algorithm to solve Wiener-Hopf equations
%
% USE: LinearPred TSIF PASOF ErrOF ARCOF M
%
% TSIF => Time Series ASCII Input File
% PASOF => Predicted & Actual Series Output File
% ErrOF => Instant NSE & NMSE errors Output File
% ARCOF => AR Coefficients Output File yields the coefficients C_0, C_1, ..., C_M of u(n+1)=-C_1*u(n)-C_2*u(n-1)...-C_M*u(n-C_M+1)
% M => Prediction Order (Process State-Vectors Dimension)
% By Cristina & Fernando De Castro October 2001
%

function LinearPred(TSIF, PASOF,ErrOF,ARCOF,M)
if nargin != 5
   disp('LinearPred: TSIF PASOF ErrOF ARCOF M');
   return
end
n=eval(M);

% read time series into S
InFp=fopen(TSIF,'rt');
S=fscanf(InFp,'%f');
fclose(InFp);

% predict S into P
[a,g]=lpc(S,n);
th=poly2th(a,[],[1]);
P=predict(S,th,1);

LenS=length(S);
LenP=length(P);
NSmpls=min(LenS,LenP);
 
% plot
k=1:NSmpls; plot(k,S,'b',k,P,'r');

% output Pred & Actual (=Observed)
OutFp=fopen(PASOF,'wt');
fprintf(OutFp,'%u\t%8.6g\t%8.6g\n',[k-1;P';S']);
fclose(OutFp);

ErrEn=0;
PredEn=0;

% NMSE calculation 
for k=2:NSmpls
ErrEn=ErrEn+(S(k)-P(k))^2;
PredEn=PredEn+(S(k)-S(k-1))^2;
NMSE(k-1)=ErrEn/PredEn;
end

% NSE calculation
for k=2:NSmpls
NSE(k-1)=(S(k)-P(k))^2/S(k)^2;
end

str=sprintf('\nNMSE=%8.6g\n',NMSE(NSmpls-1)); disp(str);
 
%disp('AR Coefficients:');disp(a);

k=2:NSmpls;

% output NSE & NMSE
OutFp=fopen(ErrOF,'wt');
fprintf(OutFp,'%u\t%8.6g\t%8.6g\n',[k-1;NSE;NMSE]);
%fprintf(OutFp,'%8.6g\t%8.6g\n',[NSE;NMSE]);
%fprintf(OutFp,'%u\n',[k-1]);
fclose(OutFp);

% output AR Coefficients 
OutFp=fopen(ARCOF,'wt');
fprintf(OutFp,'%8.6g\n',a');
fclose(OutFp);
